package index;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.FileSplit;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

@SuppressWarnings("deprecation")
public class Indexer {

	public static class LineIndexMapper extends MapReduceBase implements
			Mapper<LongWritable, Text, Text, Text> {

		private final static Text word = new Text();
		private final static Text location = new Text();

		public void map(LongWritable key, Text val,
				OutputCollector<Text, Text> output, Reporter reporter)
				throws IOException {

			FileSplit fileSplit = (FileSplit) reporter.getInputSplit();
			String fileName = fileSplit.getPath().getName();
			location.set(fileName);

			String line = val.toString();
			String[] words = line.toLowerCase().split(
					"(\\s|\\t|\\.|\\,|\\:|\\-|\\?|\\!|\\(|\\)|\\=|\\/|\\\\)");
			for (String wrd : words) {
				if (wrd.equals(""))
					continue;
				word.set(wrd);
				output.collect(word, location);
			}
		}
	}

	public static class LineIndexReducer extends MapReduceBase implements
			Reducer<Text, Text, Text, Text> {

		public void reduce(Text key, Iterator<Text> values,
				OutputCollector<Text, Text> output, Reporter reporter)
				throws IOException {

			Set<String> files = new HashSet<String>();
			while (values.hasNext()) {

				files.add(values.next().toString());
			}

			boolean first = true;
			StringBuilder toReturn = new StringBuilder();
			for (String file : files) {
				if (!first)
					toReturn.append(" ");
				first = false;
				toReturn.append(file);
			}

			output.collect(key, new Text(toReturn.toString()));
		}
	}

	public static void run() throws IOException {
		JobClient client = new JobClient();
		JobConf conf = new JobConf(Indexer.class);

		conf.setJobName("Indexer");

		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(Text.class);

		FileInputFormat.addInputPath(conf, new Path("Files"));
		FileOutputFormat.setOutputPath(conf, new Path("Index"));
		conf.setMapperClass(LineIndexMapper.class);
		conf.setReducerClass(LineIndexReducer.class);

		client.setConf(conf);

		Path indexPath = new Path("Index");
		boolean isIndexDeleted;
		FileSystem hdfs = FileSystem.get(new Configuration());
		if (!hdfs.exists(indexPath)) {
			isIndexDeleted = true;
		} else {
			isIndexDeleted = hdfs.delete(indexPath, true);
		}

		if (isIndexDeleted) {
			try {
				JobClient.runJob(conf);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}