package utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;

public class RPNUtils {

	public static String infixToRPN(String input) {
		if (input == null)
			return "";
		char[] in = input.toCharArray();
		Stack<Character> stack = new Stack<Character>();
		StringBuilder out = new StringBuilder();

		for (int i = 0; i < in.length; i++)
			switch (in[i]) {
			case '+':
				while (!stack.empty() && (stack.peek() == '*')) {
					out.append(' ');
					out.append(stack.pop());
				}
				out.append(' ');
				stack.push(in[i]);
				break;
			case '*':
				out.append(' ');
				stack.push(in[i]);
				break;
			case '(':
				stack.push(in[i]);
				break;
			case ')':
				while (!stack.empty() && stack.peek() != '(') {
					out.append(' ');
					out.append(stack.pop());
				}
				stack.pop();
				break;
			default:
				out.append(in[i]);
				break;
			}

		while (!stack.isEmpty())
			out.append(' ').append(stack.pop());

		return out.toString();
	}

	public static List<String> parseQuery(String query) {
		String lowercaseQuery = query.toLowerCase();
		String[] splitQuery = lowercaseQuery.split("(\\s|\\+|\\*|\\(|\\))");
		List<String> queryList = new ArrayList<String>();
		for (String word : splitQuery) {
			if (word.equals(""))
				continue;
			queryList.add(word.trim());
		}
		return queryList;
	}

	public static Set<String> compute(Map<String, Set<String>> cache,
			String[] rpnQuery) {
		Stack<Set<String>> result = new Stack<Set<String>>();
		for (int i = 0; i < rpnQuery.length; ++i) {
			if (rpnQuery[i].equals("*")) {
				Set<String> top = result.pop();
				result.peek().retainAll(top);
			} else if (rpnQuery[i].equals("+")) {
				Set<String> top = result.pop();
				result.peek().addAll(top);
			} else {
				result.add(cache.get(rpnQuery[i]));
			}

		}
		return result.pop();
	}

}
